@@ -2,25 +2,17 @@ import requests
 import json
 from easysettings import EasySettings
 from pathlib import Path
-import tensorflow as tf
 import shutil
+import dill
 import os
-import zipfile
 from joblib import dump
+import zipfile
+from .utils import zipdir, getApiKey, generate_requirements_file, auth, get_model, new_version, deploy_version, find_version
 
 api_host = 'https://api.inferrd.com'
 
 settings = EasySettings(str(Path.home()) + "/.inferrd.conf")
 
-def zipdir(path, ziph):
-    # ziph is zipfile handle
-    for root, dirs, files in os.walk(path):
-        for file in files:
-            filePath = os.path.join(root, file)
-            inZipPath = os.path.relpath(os.path.join(root, file), os.path.join(path, '..'))
-            # remove base folder
-            ziph.write(filePath, inZipPath.replace('inferrd-model/', ''))
-
 __all__ = [
     'indextools',
     'doctools'
@@ -29,43 +21,11 @@ __all__ = [
 def __main__():
   print('Hi')
 
-def auth(key):
-  settings.set('api_key', key)
-  settings.save()
-
-def get_model(name):
-  api_key = settings.get('api_key')
-
-  r = requests.get(api_host + '/service/find/' + name, headers={'Content-Type': 'application/json', 'Authorization': 'Token ' + api_key})
-
-  return r.json()
-
-def new_version(modelId):
-  api_key = settings.get('api_key')
-
-  r = requests.post(api_host + '/service/' + modelId + '/versions', headers={'Content-Type': 'application/json', 'Authorization': 'Token ' + api_key})
-
-  return r.json()
-
-def deploy_version(versionId):
-  api_key = settings.get('api_key')
-
-  r = requests.post(api_host + '/version/' + versionId + '/deploy', headers={'Content-Type': 'application/json', 'Authorization': 'Token ' + api_key})
-
-  return r.json()
-
-def find_version(modelId, name):
-  api_key = settings.get('api_key')
-
-  r = requests.get(api_host + '/service/' + modelId + '/versions/find/' + name, headers={'Content-Type': 'application/json', 'Authorization': 'Token ' + api_key})
-
-  return r.json()
-
 # arguments
 # includeFailures=True/False
 def get_requests(name, **kwargs):
   model = get_model(name)
-  api_key = settings.get('api_key')
+  api_key = getApiKey()
 
   includeFailures = False
   version = None
@@ -93,9 +53,54 @@ def get_requests(name, **kwargs):
 
   return r.json()
 
+def deploy(model, **kwargs):
+  if(getApiKey() == ''):
+    print('No api key. Use inferrd.auth() first.')
+    exit()
+
+  prediction_fn = model
+  name = kwargs['name']
+  
+  if prediction_fn is None:
+    print('Empty function model. Make sure the first argument is a function.')
+    exit()
+
+  model = get_model(name)
+
+  version = new_version(model['id'])
+
+  print('> Preping model for deployment')
+
+  if os.path.exists('./model.dill'):
+    os.remove('./model.dill')
+
+  dill.dump(prediction_fn, open('./model.dill', mode='wb'), recurse=True)
+  generate_requirements_file()
+
+  zipf = zipfile.ZipFile('model.zip', 'w', zipfile.ZIP_DEFLATED)
+  zipf.write('./model.dill', './model.dill')
+  zipf.write('./reqs.txt', './requirements.txt')
+  zipf.close()
+  
+  # upload to storage
+  print('> Uploading model')
+  f = open("./model.zip", 'rb')
+  r = requests.put(version['signedUpload'], data=f, headers={'Content-Type': 'application/zip'})
+
+  print('> Deploying version v' + str(version['number']))
+  deploy_version(version['id'])
+
+  os.remove('./model.zip')
+  os.remove('./model.dill')
+
+  print('Your model is now deploying! Try it out using this code:\n')
+  print('inferrd.get(\'{0}\')([1,2])'.format(name))
+  return version['number']
+
+
 # ------ TENSORFLOW
 def deploy_tf(tf_model, name):
-  if(settings.get('api_key') == ''):
+  if(getApiKey() == ''):
     print('No api key. Use inferrd.auth() first.')
     exit()
 
@@ -112,6 +117,8 @@ def deploy_tf(tf_model, name):
   if os.path.exists('./inferrd-model'):
     shutil.rmtree('./inferrd-model')
 
+  import tensorflow as tf
+
   tf.saved_model.save(tf_model, './inferrd-model')
     
   print('> Zipping model for upload')
@@ -139,7 +146,7 @@ def deploy_tf(tf_model, name):
 
 # ------ SCIKIT
 def deploy_scikit(scikit_model, name):
-  if(settings.get('api_key') == ''):
+  if(getApiKey() == ''):
     print('No api key. Use inferrd.auth() first.')
     exit()
 
@@ -183,7 +190,7 @@ def deploy_scikit(scikit_model, name):
 
 # ------ SPACY
 def deploy_spacy(nlp_model, name):
-  if(settings.get('api_key') == ''):
+  if(getApiKey() == ''):
     print('No api key. Use inferrd.auth() first.')
     exit()
 
